#include <nitro.h>
#include <ninet/ip.h>
#include <dwc.h>

#include "main.h"
#include "init.h"
#include "utility.h"
#include "registerfriend.h"
#include "transport.h"
#include "dbs.h"

#include "screen.h"
#include "logon.h"

//----------------------------------------------------------------------------
// define
//----------------------------------------------------------------------------
#define GAME_NUM_MATCH_KEYS 3      //  Number of additional keys for matchmaking

//----------------------------------------------------------------------------
// variable
//----------------------------------------------------------------------------
extern DWCUserData stUserData; // 

extern GameSequence gameSeqList[GAME_MODE_NUM];
extern GameControl stGameCnt;  //  Game control data structure
extern KeyControl stKeyCnt;    //  Key input control structure

// Additional key data structure for peer matchmaking without a friend specification
static GameMatchExtKeys stMatchKeys[GAME_NUM_MATCH_KEYS] = { 0, };

// Sample data set for use as GameSpy status data
static const char gsStatusSample[16] = { 10, 9, 8, 0, 0, 1, 2, 3, };

static int stNumEntry  = 2;    //  Specify the number of players making up the network
static int stServerIdx = 0;    //  Friend list index for the server DS to be connected to


static void SetGameMatchKeys(void);

GameMode StopIPMain(void);


//callback prototype
static void ConnectToAnybodyCallback(DWCError error, BOOL cancel, void* param);
static void ConnectToFriendsCallback(DWCError error, BOOL cancel, void* param);
static void SetupGameServerCallback(DWCError error,
                                    BOOL cancel,
                                    BOOL self,
                                    BOOL isServer,
                                    int index,
                                    void* param);
static void ConnectToGameServerCallback(DWCError error,
                                        BOOL cancel,
                                        BOOL self,
                                        BOOL isServer,
                                        int index,
                                        void* param);
static void NewClientCallback(int index, void* param);
static int  EvaluateAnybodyCallback(int index, void* param);
static int  EvaluateFriendCallback(int index, void* param);


/*---------------------------------------------------------------------------*
  Additional key set function for matchmaking using game definitions
 *---------------------------------------------------------------------------*/
static void SetGameMatchKeys(void)
{
    int ranking = (int)((OS_GetTick()*1592653589UL+453816691UL) & 0xff);

    // Ranking set as integer key
    stMatchKeys[0].isStr  = 0;
    strcpy(stMatchKeys[0].keyStr, "game_rank");
    stMatchKeys[0].ivalue = ranking;
    stMatchKeys[0].keyID  =
        DWC_AddMatchKeyInt(stMatchKeys[0].keyID,
                           stMatchKeys[0].keyStr,
                           &stMatchKeys[0].ivalue);

    // Evaluation type used for matchmaking set as text string key
    stMatchKeys[1].isStr  = 1;
    strcpy(stMatchKeys[1].keyStr, "game_mtype");
    strcpy(stMatchKeys[1].svalue, "eval_rank");
    stMatchKeys[1].keyID  =
        DWC_AddMatchKeyString(stMatchKeys[1].keyID,
                              stMatchKeys[1].keyStr,
                              stMatchKeys[1].svalue);

    // Set one more integer key for the time being
    stMatchKeys[2].isStr  = 0;
    strcpy(stMatchKeys[2].keyStr, "game_manner");
    stMatchKeys[2].ivalue = ranking%11;
    stMatchKeys[2].keyID  =
        DWC_AddMatchKeyInt(stMatchKeys[2].keyID,
                           stMatchKeys[2].keyStr,
                           &stMatchKeys[2].ivalue);
}


/*---------------------------------------------------------------------------*
  Callback function for additional display after login
 *---------------------------------------------------------------------------*/
void LogonModeDispCallback(int curIdx, void* param)
{
#pragma unused(param)

    // Display the pointer to the number of player entries excluding oneself
    dbs_CPrintf(NULL, "\nnumber of entry = ");
    if (stNumEntry == 2){
        dbs_CPrintf(NULL, "   %d >>\n", stNumEntry);
    }
    else if (stNumEntry == DWC_MAX_CONNECTIONS){
        dbs_CPrintf(NULL, "<< %d\n", stNumEntry);
    }
    else {
        dbs_CPrintf(NULL, "<< %d >>\n", stNumEntry);
    }

    if (curIdx == 3){
        // when connecting to the server-client matchmaking server DS
        // the friend list index is also shown
        dbs_CPrintf(NULL, "GameServerIndex = ");
        if (DWC_GetNumFriend(stGameCnt.friendList.keyList, GAME_MAX_FRIEND_LIST) == 1){
            dbs_CPrintf(NULL, "   %d\n", stServerIdx);
        }
        else if (stServerIdx == 0){
            dbs_CPrintf(NULL, "   %d >>\n", stServerIdx);
        }
        else if (stServerIdx == GAME_MAX_FRIEND_LIST-1){
            dbs_CPrintf(NULL, "<< %d\n", stServerIdx);
        }
        else {
            dbs_CPrintf(NULL, "<< %d >>\n", stServerIdx);
        }
    }
}

/*---------------------------------------------------------------------------*
  new connection client notification callback function for server-client matchmaking
 *---------------------------------------------------------------------------*/
static void NewClientCallback(int index, void* param)
{
#pragma unused(param)

    // Controller operations are disabled until connection processing for newly connected clients ends
    // controller operations are prohibited
    stGameCnt.blocking = TRUE;
    
    OS_TPrintf("New client started to connect.\n");
    if (index != -1){
        OS_TPrintf("He is my friend [%d].\n", index);
    }
    else {
        OS_TPrintf("He is not my friend.\n");
    }
}


/*---------------------------------------------------------------------------*
  callback function for evaluating players when using peer matching without a friend specification
 *---------------------------------------------------------------------------*/
static int  EvaluateAnybodyCallback(int index, void* param)
{
#pragma unused(param)
    const char* matchType = DWC_GetMatchStringValue(index, stMatchKeys[1].keyStr, "NONE");
    int rank;
    int diff;
    
    if (!strncmp(matchType, "eval_rank", strlen("eval_rank"))){
        // When the other party also sets "eval_rank" as the matchmaking type
        rank = DWC_GetMatchIntValue(index, stMatchKeys[0].keyStr, -1);
        if (rank == -1){
            // matchmaking is not performed without a "game_rank" key
            return 0;
        }

        diff = (stMatchKeys[0].ivalue > rank) ? stMatchKeys[0].ivalue-rank : rank-stMatchKeys[0].ivalue;

        OS_TPrintf("Player[%d]'s rank is %d (mine %d). %d point.\n",
                   index, rank, stMatchKeys[0].ivalue, 200-diff);

#if 0
        // Set this to 1 if you want to perform an evaluation test
        // Assign point score such that the smaller the difference in rank as compared to the local host, the higher the points assigned, with 200 points being the maximum.
        // Do not perform matchmaking for scores of 200 or higher (Return values of 0 and lower)
             return 200-diff;

#else
        // Goes through this part because it's usually a hassle those times that matchmaking isn't performed
        return 300-diff;
#endif
    }
    else if (!strncmp(matchType, "NONE", strlen("NONE"))){
        // If there is no "game_mtype" key, perform matchmaking evenly
        return 1;
    }

    return 1;
}


/*---------------------------------------------------------------------------*
  callback function for evaluating players when using peer matching with a friend specification
 *---------------------------------------------------------------------------*/
static int  EvaluateFriendCallback(int index, void* param)
{
#pragma unused(param)
    const char* matchType = DWC_GetMatchStringValue(index, stMatchKeys[1].keyStr, "NONE");
    
    if (!strncmp(matchType, "eval_rank", strlen("eval_rank"))){
        // When the other party also sets "eval_rank" as the matchmaking type
        return 1;  //  matchmaking is OK
    }
    else {
        // matchmaking is not performed without a "game_mtype" key
        return 0;
    }
}

/*---------------------------------------------------------------------------*
  Main function after login
 *---------------------------------------------------------------------------*/
GameMode GameLogonMain(void)
{
    GameSequence *gameSeq = &gameSeqList[GAME_MODE_LOGIN];
    GameMode returnSeq = GAME_MODE_NUM;
    int curIdx = 0;
    BOOL boolRet;

    //stNumEntry  = 2;
    stServerIdx = 0;

    // Set debug report level
    DWC_SetReportLevel(DWC_REPORTFLAG_ALL);

#if 1
// Sample friend registration key display test
{
    char tmpstr[16];
    
    DWC_Acc_FriendKeyToNumericKey(DWC_CreateFriendKey(&stUserData), tmpstr);
    OS_TPrintf("friendKeyToken '%s'\n", tmpstr);
}
#endif

#if 0
// Set up test for matchmaking complete options for the number of specified players or less
{
    DWCMatchOptMinComplete optval;

    optval.valid    = 1;
    optval.minEntry = 4;
    optval.timeout  = 60000;

    if (DWC_SetMatchingOption(DWC_MATCH_OPTION_MIN_COMPLETE, &optval, 0)){
        OS_TPrintf("Failed to set matching option\n");
    }
}
#endif

    while (1){
        dbs_Print( 0, 0, "s:%d", DWC_GetMatchingState() );
        dbs_Print( 7, 0, "n:%d", DWC_GetNumConnectionHost() );
        dbs_Print( 0, 1, "w:%d", DWC_GetLinkLevel() );
        dbs_Print( 10,1, "p:%d", stGameCnt.userData.profileID );

        ReadKeyData();  //  Get key data

        DWC_ProcessFriendsMatch();  //  Update DWC communication processing

        if (DWC_GetLastError(NULL)){
            // Processing when matchmaking failed
            int err;
            DWC_GetLastError(&err);
            OS_TPrintf("LOGON : OCCUR ERROR %d:%d\n",DWC_GetLastError(NULL), err );
            ShutdownInet();  //  Disconnect network
            DWC_ClearError();
            stGameCnt.blocking = FALSE;
            returnSeq = GAME_MODE_MAIN;
            break;
        }

        if (stGameCnt.blocking){
            // Prohibit key operations during DWC processing (only for cancellations)
            if (stKeyCnt.trg & PAD_BUTTON_B){
                // Cancel matchmaking using the B button
                boolRet = DWC_CancelMatching();
                if (!boolRet){
                    OS_TPrintf("Now unable to cancel matching.\n");
                }
            }

            dbs_DemoUpdate();
            OS_WaitIrq(TRUE, OS_IE_V_BLANK);
            dbs_DemoLoad();
            // Check for stack overflow
            OS_CheckStack(OS_GetCurrentThread());
            continue;
        }

        // Escapes the loop if the next mode to enter has been set
        if (returnSeq != GAME_MODE_NUM) break;

        ////////// Process the following key operations
        if (stKeyCnt.trg & PAD_BUTTON_A){
            // Select the menu with the A button
            returnSeq = gameSeq->menuList[curIdx].mode;
            stGameCnt.blocking = gameSeq->menuList[curIdx].blocking;

            switch (curIdx){
            case 0:  //  Call peer matchmaking without a friend specification
                if (!(stKeyCnt.cont & PAD_BUTTON_X)){
                    // Supplemental key data settings for matchmaking if the X button hasn't been pressed
                    SetGameMatchKeys();
                }
                
                DWC_ConnectToAnybodyAsync((u8)stNumEntry,
                                          "game_mtype = 'eval_rank'",
                                          ConnectToAnybodyCallback,
                                          &returnSeq,
                                          EvaluateAnybodyCallback,
                                          NULL);
                break;
            case 1:  //  Call peer matchmaking with friend specification
                if (!(stKeyCnt.cont & PAD_BUTTON_X)){
                    // Supplemental key data settings for matchmaking if the X button hasn't been pressed
                    SetGameMatchKeys();
                }

                // If NULL is specified for the friendIdxList argument, a search is made for all members on the friend roster
                // Search all friends on friend list.
                DWC_ConnectToFriendsAsync(NULL,
                                          0,
                                          (u8)stNumEntry,
                                          TRUE,//FALSE,
                                          ConnectToFriendsCallback,
                                          &returnSeq,
                                          EvaluateFriendCallback,
                                          NULL);
                break;
            case 2:  //  begin server-client matchmaking as the server DS
                DWC_SetupGameServerAsync((u8)stNumEntry,
                                         SetupGameServerCallback,
                                         &returnSeq,
                                         NewClientCallback,
                                         NULL);
                break;
            case 3:  //  begin server-client matchmaking as the client DS
                DWC_ConnectToGameServerAsync(stServerIdx,
                                             ConnectToGameServerCallback,
                                             &returnSeq,
                                             NewClientCallback,
                                             NULL);
                break;
            case 4:  //  Logout
                ShutdownInet();  //  Disconnect network
				StopIPMain();
                break;
            default:
                break;
            }
        }
        else if (stKeyCnt.trg & PAD_BUTTON_Y){
            if (!(stKeyCnt.cont & PAD_BUTTON_X)){
                // As long as the X button has not been pressed,
                // the current status of friends can be gotten and displayed (statusString) using the Y button
                DispFriendList(TRUE);
                OS_TPrintf("\n");
            }
            else {
                // As long as the X button is being pressed,
                // the current status of friends can be gotten and displayed (statusData) using the Y button
                DispFriendListData();
                OS_TPrintf("\n");
            }
        }
        else if (stKeyCnt.trg & PAD_BUTTON_SELECT){
#if 1  //  When testing GameSpy Status settings
            if (!(stKeyCnt.cont & PAD_BUTTON_X)){
                // As long as the X button has not been pressed,
                // the GameSpy Status string can be set using the select button
                if (DWC_SetGsStatusString("SampleString-00")){
                    OS_TPrintf("Set GsStatusString.\n");
                }
                else {
                    OS_TPrintf("Failed to set GsStatusString.\n");
                }
            }
            else {
                // As long as the X button is being pressed,
                // the GameSpy Status data can be set using the select button
                if (DWC_SetGsStatusData(gsStatusSample, sizeof(gsStatusSample))){
                    OS_TPrintf("Set GsStatusData.\n");
                }
                else {
                    OS_TPrintf("Failed to set GsStatusData.\n");
                }
            }

#else  //  When testing saving to the storage server
            // Save the test key to the storage server using the select button
            char keyvalues[12];
            BOOL boolResult;

            sprintf(keyvalues, "\\test_key\\%u", OS_GetTick());

#if 1
            // Register as a public key
            boolResult = DWC_SavePublicDataAsync(keyvalues, NULL);
#else
            // Register as a private key
            boolResult = DWC_SavePrivateDataAsync(keyvalues, NULL);
#endif

            if (boolResult){
                OS_TPrintf("Saved '%s'.\n", keyvalues);
            }
            else {
                OS_TPrintf("Can't save to storage server.\n");
            }
#endif
        }
        else if (stKeyCnt.trg & PAD_BUTTON_START){
#if 1  //  When testing GameSpy Status settings
            char statusData[DWC_GS_STATUS_STRING_LEN];
            
            if (!(stKeyCnt.cont & PAD_BUTTON_X)){
                // As long as the X button has not been pressed,
                // the GameSpy Status string can be gotten using the Start button
                if (DWC_GetGsStatusString(statusData)){
                    OS_TPrintf("Got my GsStatusString : %s\n", statusData);
                }
                else {
                    OS_TPrintf("Failed to get GsStatusString.\n");
                }
            }
            else {
                // As long as the X button is being pressed,
                // the GameSpy Status data can be gotten using the Start button
                int size, i;

                size = DWC_GetGsStatusData(statusData);
                if (size != -1){
                    OS_TPrintf("Got my GsStatusData\n");
                    for (i = 0; i < size; i++){
                        OS_TPrintf("%02d, ", statusData[i]);
                    }
                    OS_TPrintf("\n");
                }
                else {
                    OS_TPrintf("Failed to get GsStatusData.\n");
                }
            }

#else  //  When testing saving to the storage server
            // Load test data from the storage server using the Start button
#if 0
            int profileID;
#endif
            BOOL boolResult;

#if 1
            // Load local host data
            boolResult = DWC_LoadOwnDataAsync("\\test_key", NULL);
#else
            // Load host data on the friend roster
            boolResult = DWC_LoadOthersDataAsync("\\test_key", stServerIdx, NULL);
#endif
            if (!boolResult) OS_TPrintf("Can't load from storage server.\n");
#endif
        }
        else if (stKeyCnt.trg & PAD_KEY_UP){
            // Move the cursor using up on the crosskey
            curIdx--;
            if (curIdx < 0) curIdx = gameSeq->numMenu-1;
            // Redisplay menu list
            DispMenuMsgWithCursor(gameSeq, curIdx, NULL);
        }
        else if (stKeyCnt.trg & PAD_KEY_DOWN){
            // Move the cursor using down on the crosskey
            curIdx++;
            if (curIdx >= gameSeq->numMenu) curIdx = 0;
            // Redisplay menu list
            DispMenuMsgWithCursor(gameSeq, curIdx, NULL);
        }
        else if (stKeyCnt.trg & PAD_KEY_LEFT){
            // Decrement the number of entries using the left button on the crosskey
            if (stNumEntry > 2){
                stNumEntry--;
                // Redisplay menu list
                DispMenuMsgWithCursor(gameSeq, curIdx, NULL);
            }
        }
        else if (stKeyCnt.trg & PAD_KEY_RIGHT){
            // Increment the number of entries using the right button on the crosskey
            if (stNumEntry < DWC_MAX_CONNECTIONS){
                stNumEntry++;
                // Redisplay menu list
                DispMenuMsgWithCursor(gameSeq, curIdx, NULL);
            }
        }
        else if (stKeyCnt.trg & PAD_BUTTON_L){
            // select the friend list index with the LR button for the
            // server-client matchmaking server DS
            if (curIdx == 3){
                if (stServerIdx > 0){
                    stServerIdx--;
                    // Redisplay menu list
                    DispMenuMsgWithCursor(gameSeq, curIdx, NULL);
                }
            }
        }
        else if (stKeyCnt.trg & PAD_BUTTON_R){
            // select the friend list index with the LR button for the
            // server-client matchmaking server DS
            if (curIdx == 3){
                if (stServerIdx < GAME_MAX_FRIEND_LIST-1){
                    stServerIdx++;
                    // Redisplay menu list
                    DispMenuMsgWithCursor(gameSeq, curIdx, NULL);
                }
            }
        }
        ////////// Key operation processing ends here

        dbs_DemoUpdate();
        OS_WaitIrq(TRUE, OS_IE_V_BLANK);
        dbs_DemoLoad();
        // Check for stack overflow
        OS_CheckStack(OS_GetCurrentThread());
    }

    return returnSeq;
}


/*---------------------------------------------------------------------------*
  Callback function for connecting without a friend specification
 *---------------------------------------------------------------------------*/
static void ConnectToAnybodyCallback(DWCError error, BOOL cancel, void* param)
{

    if (error == DWC_ERROR_NONE){
        if (!cancel){
            // Establish connection with unknown players
            OS_TPrintf("Succeeded to connect to anybody\n\n");

            // Set the receive buffer
            SetRecvBuffer();
        }
        else {
            OS_TPrintf("Canceled matching.\n");
            // Returns to the status after login
            *(GameMode *)param = GAME_MODE_LOGIN;
        }
    }
    else {
        DispErrorMessage();
        OS_TPrintf("Failed to connect to anybody. %d\n\n", error);

        // One more time from the beginning
        *(GameMode *)param = GAME_MODE_MAIN;
    }

    stGameCnt.blocking = FALSE;  //  Cancel blocking
}


/*---------------------------------------------------------------------------*
  Callback function for connecting with a friend specification
 *---------------------------------------------------------------------------*/
static void ConnectToFriendsCallback(DWCError error, BOOL cancel, void* param)
{

    if (error == DWC_ERROR_NONE){
        if (!cancel){
            // Establish connection with friend
            OS_TPrintf("Succeeded to connect to friends\n\n");

            // Set the receive buffer
            SetRecvBuffer();
        }
        else {
            OS_TPrintf("Canceled matching.\n");
            // Returns to the status after login
            *(GameMode *)param = GAME_MODE_LOGIN;
        }
    }
    else {
        DispErrorMessage();
        OS_TPrintf("Failed to connect to friends. %d\n\n", error);

        // One more time from the beginning
        *(GameMode *)param = GAME_MODE_MAIN;
    }

    stGameCnt.blocking = FALSE;  //  Cancel blocking
}


/*---------------------------------------------------------------------------*
  server-client matchmaking server launch callback function
 *---------------------------------------------------------------------------*/
static void SetupGameServerCallback(DWCError error,
                                    BOOL cancel,
                                    BOOL self,
                                    BOOL isServer,
                                    int  index,
                                    void* param)
{
#pragma unused(isServer)

    if (error == DWC_ERROR_NONE){
        if (!cancel){
            // A new client has been added to the network
            OS_TPrintf("A friend of mine joined the game.\n");
            OS_TPrintf("friendListIndex = %d.\n\n", index);

            // Set the receive buffer
            SetRecvBuffer();
        }
        else {
            if (self){
                // The self has cancelled matchmaking
                OS_TPrintf("Canceled matching.\n\n");
                // Returns to the status after login
                *(GameMode *)param = GAME_MODE_LOGIN;
            }
            else {
                // the client DS canceled matchmaking
                OS_TPrintf("Client (friendListIndex = %d) canceled matching.\n\n", index);

                if (DWC_GetState() == DWC_STATE_MATCHING){
                    // Do not go to next mode as long as no one else is connected
                    *(GameMode *)param = GAME_MODE_NUM;
                    return;
                }
            }
        }
    }
    else {
        // Error occurred
        DispErrorMessage();
        OS_TPrintf("Game server error occured. %d\n\n", error);

        // One more time from the beginning
        *(GameMode *)param = GAME_MODE_MAIN;
    }

    stGameCnt.blocking = FALSE;  //  Cancel blocking
}


/*---------------------------------------------------------------------------*
  server-client matchmaking server connection callback function
 *---------------------------------------------------------------------------*/
static void ConnectToGameServerCallback(DWCError error,
                                        BOOL cancel,
                                        BOOL self,
                                        BOOL isServer,
                                        int  index,
                                        void* param)
{

    if (error == DWC_ERROR_NONE){
        if (!cancel){
            if (self){
                // If the local host has succeeded in connecting to the network established
                // by the server DS
                OS_TPrintf("Succeeded to join the game.\n\n");
            }
            else {
                // If a new client has been added to the network
                OS_TPrintf("New client joined the game.\n");
                if (index != -1){
                    OS_TPrintf("he is my friend [%d].\n\n", index);
                }
                else {
                    OS_TPrintf("he is not my friend.\n\n");
                }
            }

            // Set the receive buffer
            SetRecvBuffer();
        }
        else {
            if (self){
                // The self has cancelled matching
                OS_TPrintf("Canceled matching.\n\n");
                // Returns to the status after login
                *(GameMode *)param = GAME_MODE_LOGIN;
            }
            else {
                if (isServer){
                    // the server DS canceled matchmaking.
                    OS_TPrintf("GameServer canceled matching.\n\n");
                    // Matchmaking is ended and the status returns to that after login
                    *(GameMode *)param = GAME_MODE_LOGIN;
                }
                else {
                    // Another client DS has cancelled matchmaking
                    OS_TPrintf("Host (friendListIndex = %d) canceled matching.\n\n", index);
                }
            }
        }
    }
    else {
        // Error occurred
        DispErrorMessage();
        OS_TPrintf("Failed to join the game. %d\n\n", error);

        // One more time from the beginning
        *(GameMode *)param = GAME_MODE_MAIN;
    }

    stGameCnt.blocking = FALSE;  //  Cancel blocking
}



