//
//  Sample program for phone mode
//

#include <nitro.h>
#include <ninet/ip.h>
#include <dwc.h>

#include "main.h"
#include "init.h"
#include "dbs.h"
#include "utility.h"
#include "screen.h"

#include "phone.h"
#include "sound.h"
#include "vct.h"

#include "vct_menu.h"
#include "demo_util.h"
#include "demo_menu.h"

#ifdef SDK_DEBUG
#include "debug_menu.h"
#endif

//
//  Global variables.
//
extern GameSequence gameSeqList[GAME_MODE_NUM];
extern GameControl stGameCnt;
extern KeyControl stKeyCnt;

//
//  Static variables.
//
static VCTSession sSession[2];

static app_info sAppInfo;

extern KeyControl stKeyCnt;
extern GameControl stGameCnt;

static Menu *sMenu = NULL;

//
//  Function prototypes.
//
static void         ClearSession(VCTSession *session);
static Menu*        InitMenu();
static void         DestroyMenu();

/////////////////////////////////////////////////////////////////////////////////////
//
//  for statistics of function cost.
//
#ifdef SDK_PROFILE_FUNCTIONCOST

#include "cost.h"

#define MYCOSTBUFFER_SIZE 0x30000
#define MYSTATBUFFER_SIZE 0x3000

static u32 myCostBuffer[MYCOSTBUFFER_SIZE / sizeof(u32)];
static u32 myStatBuffer[MYSTATBUFFER_SIZE / sizeof(u32)];

#endif

/////////////////////////////////////////////////////////////////////////////////////
//
//  A button down callback.
//
static void AButtonDown(u8 aid)
{
	int ret;

    if (sAppInfo.session == NULL) {
        VCTSession *session = VCT_CreateSession(aid);
        if (session == NULL) {
            //  Return NULL when the exhausted session or the aid is self id.
            OS_TPrintf("Can't create session!\n");
            return;
        }
        
        ret = VCT_Request(session, VCT_REQUEST_INVITE);
        if (ret != VCT_ERROR_NONE) {
            OS_TPrintf("Can't request Invite [%d]\n", ret);
            VCT_DeleteSession(session);
        }
        else {
            sAppInfo.session = session;
            PlaySE(SE_RINGOUT);
        }
    }
    else {
        switch (sAppInfo.session->state) {
        case VCT_STATE_INCOMING:
            //  Incoming call, return OK.
            //
            ret = VCT_Response(sAppInfo.session, VCT_RESPONSE_OK);
            if (ret != VCT_ERROR_NONE) {
                OS_TPrintf("Can't send response OK [%d]\n", ret);
            }
            VCT_StartStreaming(sAppInfo.session);
            BroadCastStatus(DEMO_STATUS_TALKING);
            StopSE();
            break;

        default:
            //  Do nothing.
            //
            break;
        }
    }
}

/////////////////////////////////////////////////////////////////////////////////////
//
//  B Buttton down callback.
//
static void BButtonDown()
{
    int ret;

    if (sAppInfo.session == NULL) {
        //  Do nothing when the current session is null.
        //
        return;
    }
    
    switch (sAppInfo.session->state) {
    case VCT_STATE_INCOMING:
        //
        //  Decline incoming call.
        //
        ret = VCT_Response(sAppInfo.session, VCT_RESPONSE_DECLINE);
        if (ret != VCT_ERROR_NONE) {
            OS_TPrintf("Can't send response Decline [%d]\n", ret);
        }
        StopSE();
        ClearSession(sAppInfo.session);
        break;


    case VCT_STATE_OUTGOING:
        //
        //  Cancel outgoing call.
        //
        ret = VCT_Request(sAppInfo.session, VCT_REQUEST_CANCEL);
        if (ret != VCT_ERROR_NONE) {
            OS_TPrintf("Can't send request CANCEL [%d]\n", ret);
        }
        StopSE();
        ClearSession(sAppInfo.session);
        break;

    case VCT_STATE_TALKING:
        //
        //  Disconnect call.
        //
        ret = VCT_Request(sAppInfo.session, VCT_REQUEST_BYE);
        if (ret != VCT_ERROR_NONE) {
            OS_TPrintf("Cna't send request BYE [%d]\n", ret);
        }
        //
        //
        VCT_StopStreaming(sAppInfo.session);
        BroadCastStatus(DEMO_STATUS_PHONE);
        break;
    }
}

/////////////////////////////////////////////////////////////////////////////////////
//
//  libVCT event callback function.
//
static void VoiceChatEventCallback(u8 aid, VCTEvent event, VCTSession *session, void *data)
{
#pragma unused(aid, data)
    
    int ret;

    switch (event) {
    case VCT_EVENT_INCOMING:
        if (sAppInfo.session) {
            OS_TPrintf("Return busy to %d\n", session->aid);
            ret = VCT_Response(session, VCT_RESPONSE_BUSY_HERE);
            VCT_DeleteSession(session);
            break;
        }
            
        OS_TPrintf("Invite From %d\n", session->aid);
        PlaySE(SE_RINGIN);
        sAppInfo.session = session;
        break;

    case VCT_EVENT_RESPONDBYE:
        OS_TPrintf("Bye From %d\n", session->aid);
        ret = VCT_Response(session, VCT_RESPONSE_OK);
        if (ret != VCT_ERROR_NONE) {
            OS_TPrintf("Can't send response Ok [%d]\n", ret);
        }
        ClearSession(session);
        break;

    case VCT_EVENT_DISCONNECTED:
        OS_TPrintf("Disconnected\n");
        ClearSession(session);
        break;

    case VCT_EVENT_CANCEL:
        OS_TPrintf("Cancel From %d\n", session->aid);
        ret = VCT_Response(session, VCT_RESPONSE_TERMINATED);
        if (ret != VCT_ERROR_NONE) {
            OS_TPrintf("Can't send response RequestTerminated [%d]\n", ret);
        }
        StopSE();
        ClearSession(session);
        break;

    case VCT_EVENT_CONNECTED:
        OS_TPrintf("200 OK From %d\n", session->aid);
        VCT_StartStreaming(session);
        BroadCastStatus(DEMO_STATUS_TALKING);
        StopSE();
        break;

    case VCT_EVENT_REJECT:
        OS_TPrintf("Reject From %d\n", session->aid);
        ClearSession(session);
        StopSE();
        break;

    case VCT_EVENT_BUSY:
    case VCT_EVENT_TIMEOUT:
    case VCT_EVENT_ABORT:
        OS_TPrintf("Clear session by '%s'\n", EventName(event));
        StopSE();
        ClearSession(session);
        break;

    default:
        OS_TPrintf(" not handler (%s)\n", EventName(event));
        break;
    }
}

/////////////////////////////////////////////////////////////////////////////////////
//
//  Clear session
//
static void ClearSession(VCTSession *session)
{
    if (session == sAppInfo.session) {
        VCT_StopStreaming(session);
        VCT_DeleteSession(session);
        sAppInfo.session = NULL;
    }
    else {
        SDK_WARNING(FALSE, "What is this session??\n");
        VCT_DeleteSession(session);
    }
    sMenu->ChangeMenu(MENU_AUDIO_INFO);
    BroadCastStatus(DEMO_STATUS_PHONE);
}

/////////////////////////////////////////////////////////////////////////////////////
//
//  phone mode main routine.
//
GameMode PhoneMain()
{
    GameSequence *gameSeq = &gameSeqList[GAME_MODE_PHONE];
    GameMode returnSeq = GAME_MODE_NUM;
    int curIdx = 0;

    InitMenu();
    DWC_SetReportLevel(0);

#ifdef SDK_PROFILE_FUNCTIONCOST
    OS_InitFunctionCost(&myCostBuffer, MYCOSTBUFFER_SIZE);
    OS_InitStatistics(&myStatBuffer, MYSTATBUFFER_SIZE);
#endif

	BroadCastStatus(DEMO_STATUS_PHONE);
    //
    //  Initialize libVCT
    //
    VCTConfig config;
    
    config.mode         = VCT_MODE_PHONE;
    config.session      = sSession;
    config.numSession   = 2;
    config.aid          = DWC_GetMyAID();
    config.callback     = VoiceChatEventCallback;
    config.userData     = NULL;
    config.audioBuffer     = gAudioBuffer;
    config.audioBufferSize = VCT_AUDIO_BUFFER_SIZE * VCT_DEFAULT_AUDIO_BUFFER_COUNT;

    SetupSound(1);

    if (!VCT_Init(&config)) {
        OS_TPrintf("ERROR: Can't initialize VoiceChat!!!\n");
        return GAME_MODE_CONNECTED;
    }

    //  Re-set connection close callback (re-set returnSeq). 
    DWC_SetConnectionClosedCallback(ConnectionClosedCallback, &returnSeq);
    
    sAppInfo.session = NULL;

    UpdateMenuList(gameSeq);
    DispMenuMsgWithCursor(gameSeq, curIdx, NULL);
    
    //  Start sound system. 
    StartSoundLoop();

    while (1) {
        dbs_Print( 0, 0, "s:%d", DWC_GetMatchingState() );
        dbs_Print( 7, 0, "n:%d", DWC_GetNumConnectionHost() );
        dbs_Print(14, 0, "a:%d", DWC_GetMyAID());
        dbs_Print( 0, 1, "w:%d", DWC_GetLinkLevel() );
        dbs_Print( 10,1, "p:%d", stGameCnt.userData.profileID );
        dbs_Print( 5, 2, "== Phone mode demo ==");

        ReadKeyData();

        DWC_ProcessFriendsMatch();
        
        if (DWC_GetLastError(NULL)){
            //  matchmaking failed... 
            ShutdownInet();
            DWC_ClearError();
            returnSeq = GAME_MODE_MAIN;
            break;
        }

        //  Call VCT_Main on each picture frame. 
        VCT_Main();

        if (returnSeq != GAME_MODE_NUM) break;

        //  Change menu screen with the SELECT button 
        if (stKeyCnt.trg & PAD_BUTTON_SELECT) {
            sMenu->GoNext();
        }
        int key = sMenu->ShowMenu();
        
        //
        // The 'key' has some data when ShowMenu() did not process.
        //
        if (key & PAD_BUTTON_A) {
            if (curIdx == gameSeq->numMenu - 1) {
                returnSeq = GAME_MODE_CONNECTED;
            }
            else {
                AButtonDown((u8)curIdx);
            }
        }
        else if (key & PAD_BUTTON_B) {
            BButtonDown();
            DispMenuMsgWithCursor(gameSeq, curIdx, NULL);
        }
        else if (key & PAD_KEY_UP) {
            curIdx--;
            if (curIdx < 0) curIdx = gameSeq->numMenu-1;
            DispMenuMsgWithCursor(gameSeq, curIdx, NULL);
        }
        else if (key & PAD_KEY_DOWN) {
            curIdx++;
            if (curIdx >= gameSeq->numMenu) curIdx = 0;
            DispMenuMsgWithCursor(gameSeq, curIdx, NULL);
        }
        else if (stKeyCnt.trg & PAD_BUTTON_DEBUG) {
            //  Trun off backlight for avoid NITRO-Emulator noise. 
            (void)PM_SetBackLight( PM_LCD_ALL, PM_BACKLIGHT_OFF );
        }
        
        //  Playback debugging beep wave. 
        SendBeep(stKeyCnt.cont & PAD_BUTTON_R);
        
        ShowTicks(sAppInfo.session ? StateName(sAppInfo.session->state) : "No session");

        //  NitroSystem sound main. 
        NNS_SndMain();

        //  Update screen. 
        dbs_DemoUpdate();

        //  Wait for V-blank interrupts. 
        OS_WaitIrq(TRUE, OS_IE_V_BLANK);

        dbs_DemoLoad();
        //  Check stack overflow. 
        OS_CheckStack(OS_GetCurrentThread());

#ifdef SDK_PROFILE_FUNCTIONCOST
        __vct_cost_count();
        OS_CalcStatistics(&myStatBuffer);
#endif
    }
    //  Delete all sessions. 
    ClearScreen();
    
    //  Terminate sound system. 
    StopSoundLoop();
    TerminateSound();
    
    //  Terminate libVCT. 
    VCT_Cleanup();

#ifdef SDK_PROFILE_FUNCTIONCOST
    OS_DumpStatistics(&myStatBuffer);
#endif    
    BroadCastStatus(DEMO_STATUS_EXIT);
    
    DestroyMenu();
    DWC_SetReportLevel(DWC_REPORTFLAG_ALL);


    return returnSeq;
}

/////////////////////////////////////////////////////////////////////////////////////
//
//  Menus 
//

static Menu* InitMenu()
{
    if (sMenu == NULL) {
        sMenu = new Menu(&sAppInfo);

        sMenu->AddItem(new AudioInfo);
        sMenu->AddItem(new VADInfo);

#ifdef SDK_DEBUG
        //
        //  Add debugging menu on SDK_DEBUG build. 
        //
        sMenu->AddItem(new AudioDebug);
#endif
    }

    return sMenu;
}

static void DestroyMenu()
{
    delete sMenu;
    sMenu = NULL;
}
