#include <nitro.h>
#include <ninet/ip.h>
#include <nitro/os.h>
#include <dwc.h>

#include "main.h"
#include "init.h"
#include "utility.h"

#include "dbs.h"
#include "screen.h"
#include "sound.h"

#include "transport.h"
#include "demo_util.h"

extern GameSequence gameSeqList[GAME_MODE_NUM];
extern GameControl stGameCnt;  //  Game control data structure
extern KeyControl stKeyCnt;    //  Key input control structure

//callback prototype
static void ConnectionClosedCallback(DWCError error,
                                     BOOL isLocal,
                                     BOOL isServer,
                                     u8 aid,
                                     int index,
                                     void* param);

/** -------------------------------------------------------------------------
  DWC Transport
  ---------------------------------------------------------------------------*/
const int		SIZE_SEND_BUFFER = 512;
const int		SIZE_RECV_BUFFER = 4 * 1024;

static u8		sSendBuffer[ SIZE_SEND_BUFFER ];
static u8		sRecvBuffer[ MAX_PLAYERS ][ SIZE_RECV_BUFFER ];

static int      stExchangeProfile = 0;

#define DEMO_EXCHANGE_MAGIC_TOKEN '_Vdm'

/** -------------------------------------------------------------------------
  Set receive buffer for DWC transport
  ---------------------------------------------------------------------------*/
void
SetRecvBuffer( void )
{
	int i;
    int j = 0;
	int connectingNum = DWC_GetNumConnectionHost()	;

    OS_TPrintf( "connection host = %d\n", connectingNum );
    
    for ( i = 0; i < connectingNum; ++i )
    {
		if ( i == DWC_GetMyAID() )
        {
            j++;
			continue;
        }

        DWC_SetRecvBuffer( (u8)i, &sRecvBuffer[i-j], SIZE_RECV_BUFFER );
    }
}

/*---------------------------------------------------------------------------*
  Receive complete callback
  ---------------------------------------------------------------------------*/

static void UserRecvCallback( u8 aid, u8* buffer, int size )
{
    BOOL flag;
    
    flag = VCT_HandleData(aid, buffer, size);

    if (flag == FALSE) {
        //
        // 
        // If flag is FALSE, the data is not VoiceChat data
        // or VoiceChat library is not initialized.
        //
        // In this sample, exchange client information.
        // 
        //
        client_info *info = (client_info*)buffer;
        if (info->magic != DEMO_EXCHANGE_MAGIC_TOKEN) {
            return;
        }

        UpdateClientInfo(aid, info);
    }
}

/*---------------------------------------------------------------------------*
  Main function after mesh connection has been completed
 *---------------------------------------------------------------------------*/
GameMode GameConnectedMain(void)
{
    GameSequence *gameSeq = &gameSeqList[GAME_MODE_CONNECTED];
    GameMode returnSeq = GAME_MODE_NUM;
    int curIdx = 0;

    // Change DWC error report level
    DWC_SetReportLevel(0);
    
    // Sets the send callback	
    DWC_SetUserRecvCallback( UserRecvCallback ); 
    
    // Set the receive callback	
    SetRecvBuffer();

    // Set the callback for closing connections
    DWC_SetConnectionClosedCallback(ConnectionClosedCallback, &returnSeq);

#if 0
	// Set the packet loss rate
	if ( DWC_GetMyAID() != 0 )
    {
		DWC_SetSendDrop( 30, 0 );
		DWC_SetRecvDrop( 30, 0 );        
    }
#endif
#if 0
    // Set the delay (unit: msec)
	if ( DWC_GetMyAID() != 0 )
    {
		DWC_SetSendDelay( 500, 0 );
		DWC_SetRecvDelay( 500, 0 );        
    }
#endif    

    (void)PM_SetBackLight( PM_LCD_ALL, PM_BACKLIGHT_ON );
    OS_TPrintf( "my aid = %d\n", DWC_GetMyAID() );

    // Exchange client information
    //
    BroadCastStatus(DEMO_STATUS_JOIN);
    
    DispMenuMsgWithCursor(gameSeq, curIdx, NULL);
    
    while (1){
        dbs_Print( 0, 0, "s:%d", DWC_GetMatchingState() );
        dbs_Print( 7, 0, "n:%d", DWC_GetNumConnectionHost() );
        dbs_Print(14, 0, "a:%d", DWC_GetMyAID());
        dbs_Print( 0, 1, "w:%d", DWC_GetLinkLevel() );
        dbs_Print(10, 1, "p:%d", stGameCnt.userData.profileID );

        ReadKeyData();  //  Get key data

        DWC_ProcessFriendsMatch();  //  Update DWC communication processing

        if (DWC_GetLastError(NULL)){
            // Processing when matchmaking failed
            ShutdownInet();  //  Disconnect network
            DWC_ClearError();
            returnSeq = GAME_MODE_MAIN;
            break;
        }

        if (stGameCnt.blocking) {
            // Prohibit key operations during DWC processing
            dbs_DemoUpdate();
            OS_WaitIrq(TRUE, OS_IE_V_BLANK);
            dbs_DemoLoad();
            // Check for stack overflow
            OS_CheckStack(OS_GetCurrentThread());
            continue;
	    }

        // Escapes the loop if the next mode to enter has been set
        if (returnSeq != GAME_MODE_NUM) break;

        ////////// Process the following key operations
        if (stKeyCnt.trg & PAD_BUTTON_A){
            // Select the menu with the A button
            returnSeq = gameSeq->menuList[curIdx].mode;
            stGameCnt.blocking = gameSeq->menuList[curIdx].blocking;

            switch (curIdx) {
            case 3:
                if(DWC_CloseConnectionsAsync() < 0) {
                    stGameCnt.blocking = 0;
                }
                
                if (DWC_GetNumConnectionHost() <= 1){
                    // Terminate immediately and do not call the callback if the number of hosts already connected is 0.
                    stGameCnt.blocking = 0;
                }
                break;

            case 4:
                DWC_CloseAllConnectionsHard();
                if (DWC_GetNumConnectionHost() <= 1) {
                    stGameCnt.blocking = 0;
                }
                break;

            default:
                break;
            }

        }
        else if (stKeyCnt.trg & PAD_KEY_UP){
            // Move the cursor using up on the crosskey
            curIdx--;
            if (curIdx < 0) curIdx = gameSeq->numMenu-1;
            // Redisplay menu list
            DispMenuMsgWithCursor(gameSeq, curIdx, NULL);
        }
        else if (stKeyCnt.trg & PAD_KEY_DOWN){
            // Move the cursor using down on the crosskey
            curIdx++;
            if (curIdx >= gameSeq->numMenu) curIdx = 0;
            // Redisplay menu list
            DispMenuMsgWithCursor(gameSeq, curIdx, NULL);
        }
        ////////// Key operation processing ends here

        dbs_DemoUpdate();
        OS_WaitIrq(TRUE, OS_IE_V_BLANK);
        dbs_DemoLoad();
        // Check for stack overflow
        OS_CheckStack(OS_GetCurrentThread());
    }

    return returnSeq;
}

