#ifndef __UTILITY_H__
#define __UTILITY_H__

#ifdef __cplusplus
extern "C" {
#endif


#include <nitro/types.h>

//----------------------------------------------------------------------------
// typedef
//----------------------------------------------------------------------------
// Controller input structure
// If a repeat setting is made for more than one key, only the same Delay Time and Pulse Time can be set for all of them
//  
typedef struct {
    u8  repeatSeq[10];    //  Key repeat step (0: Delay, 1: Pulse)
    u16 cont;             //  Original input
    u16 trg;              //  Trigger input
    u16 release;          //  Release input 
    u16 repeat;           //  Repeat trigger input
    u16 checkPushButton;  //  Flag for button whose key repeat status is to be checked
    u16 repeatDelayTime;  //  Time interval between first trigger input until start of key repeat
    u16 repeatPulseTime;  //  Key repeat interval
    u16 pad;              //  Padding
    u16 pushCount[10];    //  Measure the time a single button has been continuously pressed
} KeyControl;


//----------------------------------------------------------------------------
// function
//----------------------------------------------------------------------------
extern void InitKeyData(KeyControl *pKeyCntSrc);
extern void ReadKeyData(void);
extern void SetKeyRepeat(u16 keyFlg, u16 delay, u16 pulse);
extern void AddKeyRepeat(u16 keyFlg);

extern s32  UtilPow(s32 x, u32 n);


#ifdef __cplusplus
}
#endif

#endif  // __UTILITY_H__
