#ifndef __WC_H_
#define __WC_H_

#ifdef  __cplusplus
extern "C" {
#endif

/*===========================================================================*/

#include    <nitro/types.h>
#include    <nitro/wm.h>


/*---------------------------------------------------------------------------*
     constant definitions 
 *---------------------------------------------------------------------------*/
#define     WC_DMA_NO                   2       //  Make sure that there is no overlap with the DMA numbers for GX, etc.
#define     WC_DEFAULT_PORT             4       //  The port number (4 - 7)
#define     WC_DEFAULT_PORT_PRIO        2       //  The port priority (0 - 3)
#define     WC_DS_PORT_NUMBER           12      //  Port number for data sharing (12 - 15)


/*---------------------------------------------------------------------------*
     macro definitions 
 *---------------------------------------------------------------------------*/
#define     WC_ROUNDUP_32B( x ) ( ( x + 31 ) & 0xffffffe0 )


/*---------------------------------------------------------------------------*
     structure definitions 
 *---------------------------------------------------------------------------*/
typedef enum WCStatus
{
    WC_STATUS_READY = 0,               //   Initialization state 
    WC_STATUS_ERROR,                   //   Unrecoverable error state 
    WC_STATUS_BUSY,                    //   Busy state when in the middle of a state transition 
    WC_STATUS_PARENT,                  //   Connection completed state as parent device 
    WC_STATUS_CHILD,                   //   Connection completed state as child device 
    WC_STATUS_MAX
}
WCStatus;

//  Definition of the callback function type for single device MP mode 
typedef void (*WCCallbackFunc) (u16 aid, u16 *data, u16 length);

// Definition of the callback function type for block transmission mode  
// The WMCallback pointer is currently passed at the following points in time: 
// - At WM_StartMP completion
// - At WcSetParentData and WcSetChildData completion
// - At WM_Reset completion
typedef void (*WCBlockTransferCallbackFunc) (WMCallback *arg);

//  Callback function type definition for pictochat search 
typedef void (*WCPictoCatchCallbackFunc) (WMBssDesc *pBssDesc);


/*---------------------------------------------------------------------------*
    function definitions
 *---------------------------------------------------------------------------*/
void WcInit(const WMParentParam *pp, BOOL dsFlag, BOOL btFlag, u16 framePeriod);        //  Various initialization. Mainly to secure memory
void WcFinish(void);                   //  Restore the state before the initialization. Mainly to release memory.
void WcStartParent(void);              //  Starts the connection as the parent device
void WcStartChild(void);               //  Starts the connection as the child device
void WcEnd(void);                      //  Starts the transition to the initialized state
WCStatus WcGetStatus(void);            //  Gets the internal state of the WCStatus type
u16 WcGetAid(void);                    //  Gets the AID

// Configure the callback called when MP communications data is received.
void WcSetMpReceiveCallback(WCCallbackFunc cb);
// Configure the callback called when configuring block transmission mode  
void WcSetBlockTransferCallback(WCBlockTransferCallbackFunc cb);
// Set send data as the parent device
void WcSetParentData(const void *buf, u16 size);
// Set send data as the child device
void WcSetChildData(const void *buf, u16 size);

// After the connection is complete, proceed to data-sharing communications.  Call once per game frame.
BOOL WcStepDataSharing(const void *send, u16 *bitmap);
// Gets the pointer to the terminal data from the reception data  
u8 *WcGetSharedDataAddress(u16 aid);

// Enables the Pictochat Search feature.
void WcTurnOnPictoCatch(WCPictoCatchCallbackFunc func);
// Disables the Pictochat Search feature.
void WcTurnOffPictoCatch(void);


/*===========================================================================*/

#ifdef  __cplusplus
}       /* extern "C" */
#endif

#endif /* DEMOS_WIRELESS_SHARED_WC_H_ */

