#include <nitro.h>
#include "utility.h"


//----------------------------------------------------------------------------
// variable
//----------------------------------------------------------------------------
static KeyControl *pKeyCnt;


/*---------------------------------------------------------------------------*
  Function for initializing the controller structure
 *---------------------------------------------------------------------------*/
void InitKeyData(KeyControl *pKeyCntSrc)
{
    int i;

    pKeyCnt = pKeyCntSrc;
    pKeyCnt->cont    = 0;
    pKeyCnt->trg     = 0;
    pKeyCnt->release = 0;
    pKeyCnt->repeat  = 0;
    pKeyCnt->checkPushButton = 0;
    pKeyCnt->repeatDelayTime = 0;
    pKeyCnt->repeatPulseTime = 0;
    for (i = 0; i < 10; i++){
        pKeyCnt->pushCount[i] = 0;
        pKeyCnt->repeatSeq[i] = 0;
    }
}


/*---------------------------------------------------------------------------*
  Function for reading the controller
 *---------------------------------------------------------------------------*/
void ReadKeyData(void)
{
    u16 readData;
    int i;

    readData = PAD_Read();
    pKeyCnt->trg     = (u16)(readData & (readData ^ pKeyCnt->cont));  //  Trigger input
    pKeyCnt->release = (u16)(pKeyCnt->cont & (readData ^ pKeyCnt->cont));  //  Release
    pKeyCnt->cont    = readData;  //  Original input
    pKeyCnt->repeat  = 0;         //  Repeat input

    // Set key repeat status
    for (i = 0; i < 10; i++){
        if (pKeyCnt->cont & (pKeyCnt->checkPushButton & (1 << i))){
            pKeyCnt->pushCount[i]++;
            if (pKeyCnt->repeatSeq[i] == 0){
                // During delay interval
                if (pKeyCnt->pushCount[i] == pKeyCnt->repeatDelayTime){
                    // Generate repeat trigger input (delay)
                    pKeyCnt->repeat |= pKeyCnt->checkPushButton & (1 << i);
                    pKeyCnt->pushCount[i] = 0;
                    pKeyCnt->repeatSeq[i]++;
                }
            }
            else if (pKeyCnt->pushCount[i] == pKeyCnt->repeatPulseTime){
                // During pulse interval
                // Generate repeat trigger input (pulse)
                pKeyCnt->repeat |= pKeyCnt->checkPushButton & (1 << i);
                pKeyCnt->pushCount[i] = 0;
            }
        }
        else {
            pKeyCnt->pushCount[i] = 0;
            pKeyCnt->repeatSeq[i] = 0;
        }
    }
}


/*---------------------------------------------------------------------------*
  Function for setting key repeat
 *---------------------------------------------------------------------------*/
// Calling this function clears the key repeat setting used to this point
// As such, the key repeat setting can be cleared by calling this function with keyFlg = 0
void SetKeyRepeat(u16 keyFlg, u16 delay, u16 pulse)
{
    int i;

    pKeyCnt->repeat = 0;
    pKeyCnt->checkPushButton = keyFlg;
    pKeyCnt->repeatDelayTime = delay;
    pKeyCnt->repeatPulseTime = pulse;
    
    for (i = 0; i < 10; i++){
        pKeyCnt->pushCount[i] = 0;
        pKeyCnt->repeatSeq[i] = 0;
    }
}


/*---------------------------------------------------------------------------*
  Function for adding repeat key
 *---------------------------------------------------------------------------*/
void AddKeyRepeat(u16 keyFlg)
{
    int i;

    
    pKeyCnt->repeat &= ~keyFlg;
    pKeyCnt->checkPushButton |= keyFlg;

    for (i = 0; i < 10; i++){
        if (keyFlg & (1 << i)){
            pKeyCnt->pushCount[i] = 0;
            pKeyCnt->repeatSeq[i] = 0;
        }
    }
}


/*---------------------------------------------------------------------------*
  Power function
 *---------------------------------------------------------------------------*/
// Only positive indices can be retrieved because reciprocal numbers only become 0 when using integer arithmetic
s32 UtilPow(s32 x, u32 n)
{
    s32 r;
    
    r = 1;
    while (n){
        if (n & 1){
            r *= x;
        }
        x *= x;
        n >>= 1;
    }

    return r;
}
